<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;

class ConfigObserver implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
	const CUSTOM_OPTION_STRING_PATH = 'waymoreroutee/general/uuid';

    /**
     * @param Logger $logger
     */
    public function __construct(
        Logger $logger,
		\Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
		\Magento\Framework\App\RequestInterface $request,
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager
    ) {
        $this->logger = $logger;
		$this->configWriter = $configWriter;
		$this->_request = $request;
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
    }

    public function execute(EventObserver $observer)
    {
        $storeId = $this->_request->getParam('store', 0);
		$isEnabled = $this->helper->getIsEnabled($storeId);
		if($isEnabled) {
			$websiteId = $this->_request->getParam('website', 0);
            $uuld = '';
			$postedData = $this->_request->getPost()->get('groups');
			$uuld = $this->helper->getUuid($storeId);
			$usernameMain = trim($this->helper->getUsername($storeId));
            $postedData['general']['fields']['username']["value"];
			$passwordMain = trim($this->helper->getPassword($storeId));

			if($uuld == '' || trim($postedData['general']['fields']['username']["value"]) != $usernameMain || $passwordMain != trim($postedData['general']['fields']['password']["value"])) {
			    $enable = $postedData['general']['fields']['enable'];
                $username = $postedData['general']['fields']['username'];
                $password = $postedData['general']['fields']['password'];
                $webVersion = $this->_objectManager->get('Magento\Framework\App\ProductMetadataInterface')->getVersion();
                $pluginVersion = $this->moduleResource->getDbVersion('Routee_WaymoreRoutee');
                $domain = $this->_storeManager->getStore()->getBaseUrl();
                $apiUrl = $this->helper->getApiurl().'authenticate';

                $params["username"] = (isset($username["value"]) && !is_array($username["value"]) && $username["value"] != '')?trim($username["value"]):$usernameMain;
                $params["password"] = (isset($password["value"]) && !is_array($password["value"]) && $password["value"] != '')?trim($password["value"]):$passwordMain; //$password["value"];
                $params["source"] = 'Magento';
                $params["type"] = 'E-Commerce';
                $params["version"] = $webVersion;
                $params["plugin_version"] = $pluginVersion;
                $params["domain"] = $domain;

                if($storeId > 0) {
                    $scopeId = $this->_storeManager->getStore()->getId();
                    $scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORES;
                }
                elseif($websiteId > 0) {
                    $scopeId = $this->_storeManager->getWebsite()->getId();
                    $scope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITES;
                }
                else {
                    $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT;
                    $scopeId = 0;
                }

                if($params["username"] != '' && $params["password"] != '') {
                    $this->_curl->post($apiUrl, json_encode($params));
                    $this->_curl->addHeader("Content-Type", "application/json");
                    $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
                    //response will contain the output in form of JSON string
                    $response = $this->_curl->getBody();
                    $responseArr = json_decode($response, true);
                    if($responseArr['message']=='You are unauthorized. Invalid username or password')
                    {
                        throw new \Magento\Framework\Exception\CouldNotDeleteException(__($responseArr['message']));
                    }
                    else
                    {
                        $newUuid = $responseArr['uuid'];
                        $succMsg = $responseArr['message'];
                    }
                    
                    if($newUuid == '') {
                        $newUuid = $uuld;
                    }
                    $this->configWriter->save('waymoreroutee/general/uuid',  $newUuid, $scope, $scopeId);                    
                }                
			}
			elseif($uuld != '') {
                if($storeId > 0) {
                    $scopeId = $this->_storeManager->getStore()->getId();
                    $scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORES;
                }
                elseif($websiteId > 0) {
                    $scopeId = $this->_storeManager->getWebsite()->getId();
                    $scope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITES;
                }
                else {
                    $scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT;
                    $scopeId = 0;
                }                
            }
		}
		else {
			$storeId = $this->_request->getParam('store', 0);
			$websiteId = $this->_request->getParam('website', 0);
			if($storeId > 0) {
				$scopeId = $this->_storeManager->getStore()->getId();
				$scope = \Magento\Store\Model\ScopeInterface::SCOPE_STORES;	
			}
			elseif($websiteId > 0) {
				$scopeId = $this->_storeManager->getWebsite()->getId();	
				$scope = \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITES;
			}
			else {
				$scope = ScopeConfigInterface::SCOPE_TYPE_DEFAULT;	
				$scopeId = 0;	
			}
			$this->configWriter->save('waymoreroutee/general/uuid',  '', $scope, $scopeId);            
		}
    }
}