<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Eventcartupdate implements ObserverInterface
{
	protected $_logger;

	/**
	 * @param \Psr\Log\LoggerInterface $_logger
	 */

	public function __construct(
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager,
		\Magento\Sales\Model\OrderFactory $order,	
		\Magento\Framework\UrlInterface $urlInterface,	
		Logger $_logger
	){
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_logger = $_logger;
		$this->_order = $order;
		$this->_urlInterface = $urlInterface;
	}


	public function execute(EventObserver $observer)
	{
		$isEnabled = $this->helper->getIsEnabled();
		//getStoreId
		if($isEnabled) {
			$cart = $observer->getCart();
        	$allitems =  $cart->getQuote()->getAllVisibleItems();
			$storeId = $cart->getQuote()->getStoreId();
			$uuid = $this->helper->getUuid($storeId);
			$customerId = $cart->getQuote()->getCustomerId();
			if($customerId < 1) {
				$customerId = 0;	
			}
			
			$apiUrl = $this->helper->getEventApiurl();		
			
			$data = array();
			$cartDetails = array();
			$itemOptions = array();
			$data['uuid'] = $uuid;
			$data['event'] = 'CartUpdate';
			$data['data'][]['customer_id'] = $customerId;
			foreach($allitems as $item) {
				foreach ($item->getOptions() as $option) {
					$itemOptions[] = $option['option_id'];
				}
				$cartDetails['product_id'] = $item->getProductId();		
				$cartDetails['option_id'] = implode(",", $itemOptions);		
				$cartDetails['quantity'] = $item->getQty();	
				$data['data'][]['cart_products'][] = $cartDetails;
			}
			$data['data'][]['cart_url'] = $this->_urlInterface->getUrl('checkout/cart');			
			
			$res = json_encode($data['data']);
			$result = '{"uuid": "'.$uuid.'", "event": "CartUpdate", "data":'.$res.'}';
			
			$this->_curl->post($apiUrl, $result);
			$this->_curl->addHeader("Content-Type", "application/json");
			$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
			//response will contain the output in form of JSON string
			
			$response = $this->_curl->getBody();
			$responseArr = json_decode($response, true);			
		}
	}
}