<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Customer\Model\Session;

class Eventcustomerupdate implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
	protected $_customerFactory;
	const CUSTOM_OPTION_STRING_PATH = 'waymoreroutee/general/uuid';

    /**
     * @param Logger $logger
     */
    public function __construct(
        Logger $logger,
		\Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
		\Magento\Framework\App\RequestInterface $request,
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager,
		\Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerFactory,
		Session $customerSession	
    ) {
        $this->logger = $logger;
		$this->configWriter = $configWriter;
		$this->_request = $request;
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_customerFactory = $customerFactory;
		$this->_customerSession = $customerSession;
    }

    public function execute(EventObserver $observer)
    {
		$isEnabled = $this->helper->getIsEnabled();
		if($isEnabled) {
			$customer = $this->_customerSession->getCustomer()->getId();
			$storeId = $observer->getEvent()->getStoreId();
			$uuid = $this->helper->getUuid($storeId);
			
			$apiUrl = $this->helper->getEventApiurl();			
			
			$data = array();
			$data['uuid'] = $uuid;
			$data['event'] = 'CustomerUpdate';
			$customers = array();
			
			$customers['id'] = $customer;
            $customerObj = $this->_objectManager->create('Magento\Customer\Model\Customer')->load($customer);
			$customers['lastname'] = $customerObj->getLastname();
			$customers['firstname'] = $customerObj->getFirstname();
			$customers['birthday'] = $customerObj->getDob();
			$customers['email'] = $customerObj->getEmail();
			
			$addresses = $customerObj->getAddresses();
            $addrs = array();
            if(empty($addresses))
			{
				$customers['phone'] = "";

				$custAdd = array();
				$custAdd['country'] = "";
				$custAdd['city'] = "";
				$custAdd['address1'] = "";
				$custAdd['postcode'] = "";
				$custAdd['company'] = "";   
				$addrs[] = $custAdd;             				
			}
			else
			{
				foreach($addresses as $address) {
					$custAdd = array();
					$custAdd['country'] = $address->getCountryModel()->getName();
					$custAdd['city'] = $address->getCity();
					$custAdd['address1'] = $address->getStreet()[0];
					$custAdd['postcode'] = $address->getPostcode();
					$custAdd['company'] = $address->getCompany();
	                $customers['phone'] = $address->getTelephone();
	                $addrs[] = $custAdd;
				}
			}

            $data['data']['customer'] = $customers;
            $data['data']['addresses'] = $addrs;
			
			$res = json_encode($data['data']);
			$result = '{"uuid": "'.$uuid.'", "event": "CustomerUpdate", "data":'.$res.'}';
			
			$this->_curl->post($apiUrl, $result);
			$this->_curl->addHeader("Content-Type", "application/json");
			$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
			//response will contain the output in form of JSON string
			
			$response = $this->_curl->getBody();
			$responseArr = json_decode($response, true);
		}		
    }
}