<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Eventneworder implements ObserverInterface
{
	protected $_logger;

	/**
	 * @param \Psr\Log\LoggerInterface $_logger
	 */

	public function __construct(
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager,
		\Magento\Directory\Model\CountryFactory $countryFactory,
		\Magento\Newsletter\Model\Subscriber $subscriber,
		\Psr\Log\LoggerInterface $_logger
	){
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_logger = $_logger;
		$this->_countryFactory = $countryFactory;
		$this->_subscriber= $subscriber;
	}


	public function execute(EventObserver $observer)
	{
		$isEnabled = $this->helper->getIsEnabled();
		//getStoreId
		if($isEnabled) {
			$order = $observer->getEvent()->getOrder();
			$storeId = $order->getStoreId();
			$uuid = $this->helper->getUuid($storeId);
			$customerId = $order->getCustomerId();
			
			$apiUrl = $this->helper->getEventApiurl();		
			
			$payment = $order->getPayment();
			$methodPayment = $payment->getMethodInstance();
			$paymentTitle = $methodPayment->getTitle();	
			$billingAddress = $order->getBillingAddress()->getData();
			$checkSubscriber = ($this->_subscriber->loadByEmail($order->getCustomerEmail()))?'true':'false';
			$country = $this->_countryFactory->create()->loadByCode($billingAddress['country_id']);
        	$countryNameBilling = $country->getName();
			
			$data = array();
			$orderDetails = array();
			$orderProducts = array();
			$data['uuid'] = $uuid;
			$data['event'] = 'OrderAdd';
			$data['data']['customer_id'] = $customerId;
			
			$orderDetails['order_id'] = $order->getId();
			$orderDetails['order_status'] = $order->getStatusLabel();
			$orderDetails['shipping_method'] = $order->getShippingAddress()->getShippingMethod();
			$orderDetails['payment_method'] = $paymentTitle;
			$orderDetails['total_price'] = $order->getGrandTotal();
			$orderDetails['lastname'] = $order->getCustomerLastname();
			$orderDetails['firstname'] = $order->getCustomerFirstname();
			$orderDetails['birthday'] = $order->getCustomerDob();
			$orderDetails['email'] = $order->getCustomerEmail();
			$orderDetails['company'] = $billingAddress['company'];
			$orderDetails['newsletter'] = $checkSubscriber;
			$orderDetails['website'] = 'null';
			$orderDetails['address1'] = implode(" ",$order->getBillingAddress()->getStreet(0));
			$orderDetails['address2'] = implode(" ",$order->getBillingAddress()->getStreet(1));
			$orderDetails['postcode'] = $billingAddress['postcode'];
			$orderDetails['phone'] = $billingAddress['telephone'];
			$orderDetails['phone_mobile'] = "";
			$orderDetails['city'] = $billingAddress['city'];
			$orderDetails['country'] = $countryNameBilling;
			$data['data']['order_details'] = $orderDetails;
			
			$orderItems = $order->getAllItems();
			foreach ($orderItems as $item) {
					$orderProducts['product_id'] = $item->getProductId();	
				   	$orderProducts['quantity'] = $item->getQtyInvoiced();
					$orderProducts['option_id'] = $item->getItemId();
					$data['data']['order_products'][] = $orderProducts;
			}
			
			$res = json_encode($data['data']);
			$result = '{"uuid": "'.$uuid.'", "event": "OrderAdd", "data":'.$res.'}';
			
			$this->_curl->post($apiUrl, $result);
			$this->_curl->addHeader("Content-Type", "application/json");
			$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
			//response will contain the output in form of JSON string
			
			$response = $this->_curl->getBody();
			$responseArr = json_decode($response, true);
		}
	}
}