<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Eventnewproduct implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
	protected $_customerFactory;
	const CUSTOM_OPTION_STRING_PATH = 'waymoreroutee/general/uuid';

    /**
     * @param Logger $logger
     */
    public function __construct(
        Logger $logger,
		\Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
		\Magento\Framework\App\RequestInterface $request,
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager,
		\Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollectionFactory,
		\Magento\Catalog\Model\ProductFactory $_productFactory,
		\Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry
    ) {
        $this->logger = $logger;
		$this->configWriter = $configWriter;
		$this->_request = $request;
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_productCollectionFactory = $productCollectionFactory; 
        $this->_productFactory = $_productFactory;
		$this->_stockItem = $stockRegistry;
    }

    public function execute(EventObserver $observer)
    {
		$isEnabled = $this->helper->getIsEnabled();
		if($isEnabled) {
			$product = $observer->getEvent()->getProduct();
			$storeId = $this->_storeManager->getStore()->getId();	
			$uuid = $this->helper->getUuid($storeId);
			$apiUrl = $this->helper->getEventApiurl();		
			$data = array();
			$productDetails = array();
			$productOptions = array();
			$data['uuid'] = $uuid;
			$isNew = false;
			if($product->isObjectNew()){
				$isNew = true;
			}
			if($isNew) {
				$data['event'] = 'ProductAdd';
			}
			else {
				$data['event'] = 'ProductUpdate';
			}
			
			$data['data']['product_id'] = $product->getId();
			$data['data']['name'] = $product->getName();
			$data['data']['description'] = $product->getDescription();
			$data['data']['short_description'] = $product->getShortDescription();
			
			$categories = $product->getCategoryIds(); /*will return category ids array*/
			$catNames = array();
			foreach($categories as $category){
				$cat = $this->_objectManager->create('Magento\Catalog\Model\Category')->load($category);
				$catNames[] = $cat->getName();
			}
			$data['data']['categories'] = $catNames;
			$data['data']['stock_quantity'] = $this->_stockItem->getStockItem($product->getId())->getQty();
			$data['data']['image_link'] = $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . 'catalog/product' . $product->getImage();
			$data['data']['product_link'] = $product->getProductUrl();
			$data['data']['product_options'] = $this->_objectManager->get('Magento\Catalog\Model\Product\Option')->getProductOptionCollection($product)->getData();
			
			$data['data']['price'] = $product->getPrice();
			$specialprice = $product->getSpecialPrice();
			$discount = array();
			if($specialprice > 0) {
				$specialfromdate = $product->getSpecialFromDate();
				$specialtodate = $product->getSpecialToDate();
				$discount['price'] = $specialprice ;
				$discount['date_start'] = $specialfromdate ;
				$discount['date_end'] = $specialtodate ;
				$discount['quantity'] = null ;
			}
			$data['data']['discount'] = $discount;
			
			$res = json_encode($data['data']);
			$result = '{"uuid": "'.$uuid.'", "event": "'.$data['event'].'", "data":'.$res.'}';
			
			$this->_curl->post($apiUrl, $result);
			$this->_curl->addHeader("Content-Type", "application/json");
			$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
			//response will contain the output in form of JSON string
			
			$response = $this->_curl->getBody();
			$responseArr = json_decode($response, true);
		}
		else {
		}
    }
}