<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Massapicustomers implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
	protected $_customerFactory;
	const CUSTOM_OPTION_STRING_PATH = 'waymoreroutee/general/uuid';

    /**
     * @param Logger $logger
     */
    public function __construct(
        Logger $logger,
		\Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
		\Magento\Framework\App\RequestInterface $request,
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager,
		\Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerFactory
    ) {
        $this->logger = $logger;
		$this->configWriter = $configWriter;
		$this->_request = $request;
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_customerFactory = $customerFactory;
    }

    public function execute(EventObserver $observer)
    {
		$isEnabled = $this->helper->getIsEnabled();
		if($isEnabled) {
			$uuid = $observer->getData('uuid');
			$scopeId = $observer->getData('scopeId');
			$scope = $observer->getData('scope');
			$apiUrl = $this->helper->getDataApiurl();
			$customerCollection = $this->_customerFactory->create();
			$websiteId = $scopeId;	
			
			if($scope == \Magento\Store\Model\ScopeInterface::SCOPE_STORES) {
				$websiteId = $this->_storeManager->getStore($scopeId)->getWebsiteId();	
			}
			elseif($scope == \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITES) {
				$websiteId = $scopeId;	
			}
			
			if($websiteId > 0) {
				$customerCollection = $customerCollection->getCollection()
					  ->addAttributeToSelect("*")
					  ->addAttributeToFilter("website_id", array("eq" => $websiteId)); 
			}
			
			if(!empty($customerCollection) && count($customerCollection)>0)
			{
				$c = 0;
				$i = 0;
				
				$total_users = count($customerCollection);

				$mass_data = array(
					'uuid' => $uuid,
					'data' => array(
						array(
							'name' => 'Customers',
							'description' => 'Shop Customers'
						)
					)
				);
				
				foreach($customerCollection as $customer)
				{
					if($customer->getDefaultBillingAddress())
					{
						$phone = $customer->getDefaultBillingAddress()->getTelephone();
						$country = $customer->getDefaultBillingAddress()->getCountryModel()->getName();
						$city = $customer->getDefaultBillingAddress()->getCity();
						$address1 = $customer->getDefaultBillingAddress()->getStreet()[0];
						$postcode = $customer->getDefaultBillingAddress()->getPostcode();
						$company = $customer->getDefaultBillingAddress()->getCompany();
					}
					else
					{
						$phone = '';
						$country = '';
						$city = '';
						$address1 = '';
						$postcode = '';
						$company = '';	
					}
					$mass_data['data'][0]['object'][$i] = array(
						'customer' => array(
							'id' => $customer->getId(),
							'lastname' => $customer->getLastname(),
							'firstname' => $customer->getFirstname(),
							'birthday' => $customer->getDob(),
							'email' => $customer->getEmail(),
							'phone' => $phone,
						),
						'addresses' => array(
							array(
								'country' => $country,
								'city' => $city,
								'address1' => $address1,
								'postcode' => $postcode,
								'company' => $company,
							)
						)
					);

					$i++;
					$c++;
			
					if($i == 100 || $c==$total_users)
					{
						$this->_curl->post($apiUrl, json_encode($mass_data));
						$this->_curl->addHeader("Content-Type", "application/json");
						$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
						//response will contain the output in form of JSON string
						
						$i = 0;
						$mass_data['data'][0]['object'] = array();

						if($c==$total_users)
						{
							$response = $this->_curl->getBody();
							$responseArr = json_decode($response, true);
							if(!empty($responseArr['message'])) {
								$this->_eventManager->dispatch('waymoreroutee_massapicustomer_complete', ['uuid' => $uuid, 'scopeId' => $scopeId, 'scope' => $scope]);	
							}
						}
					}
				}			
			}			
		}
		else {
		}
    }

	public function getCustomerData($requestData)
    {
		$websiteId = $this->_storeManager->getWebsite()->getId();
		$customerCollection = $this->_customerFactory->create();
		$customerCollection->addAttributeToSelect('*')->addAttributeToFilter("website_id", array("eq" => $websiteId));	
		$apiUrl = $this->helper->getDataApiurl();
		
		if(!empty($customerCollection) && count($customerCollection)>0)
		{
			$c = 0;
			$i = 0;
			
			$total_users = count($customerCollection);

			$mass_data = array(
				'uuid' => $requestData['uuid'],
				'data' => array(
					array(
						'name' => 'Customers',
						'description' => 'Shop Customers'
					)
				)
			);
			
			foreach($customerCollection as $customer)
			{
				if($customer->getDefaultBillingAddress())
				{
					$phone = $customer->getDefaultBillingAddress()->getTelephone();
					$country = $customer->getDefaultBillingAddress()->getCountryModel()->getName();
					$city = $customer->getDefaultBillingAddress()->getCity();
					$address1 = $customer->getDefaultBillingAddress()->getStreet()[0];
					$postcode = $customer->getDefaultBillingAddress()->getPostcode();
					$company = $customer->getDefaultBillingAddress()->getCompany();
				}
				else
				{
					$phone = '';
					$country = '';
					$city = '';
					$address1 = '';
					$postcode = '';
					$company = '';	
				}
				$mass_data['data'][0]['object'][$i] = array(
					'customer' => array(
						'id' => $customer->getId(),
						'lastname' => $customer->getLastname(),
						'firstname' => $customer->getFirstname(),
						'birthday' => $customer->getDob(),
						'email' => $customer->getEmail(),
						'phone' => $phone,
					),
					'addresses' => array(
						array(
							'country' => $country,
							'city' => $city,
							'address1' => $address1,
							'postcode' => $postcode,
							'company' => $company,
						)
					)
				);

				$i++;
				$c++;
		
				if($i == 100 || $c==$total_users)
				{
					$this->_curl->post($apiUrl, json_encode($mass_data));
					$this->_curl->addHeader("Content-Type", "application/json");
					$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
					//response will contain the output in form of JSON string
					
					$i = 0;
					$mass_data['data'][0]['object'] = array();

					if($c==$total_users)
					{
						$response = $this->_curl->getBody();
						$responseArr = json_decode($response, true);
						if(!empty($responseArr['message'])) {
							return "CustomerDone";
						}
					}
				}
			}			
		}
		else
		{
			return "CustomerDone";	
		}
	}
}