<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Massapiorders implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
	protected $_customerFactory;
	const CUSTOM_OPTION_STRING_PATH = 'waymoreroutee/general/uuid';

    /**
     * @param Logger $logger
     */
    public function __construct(
        Logger $logger,
		\Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
		\Magento\Framework\App\RequestInterface $request,
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager,
		\Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerFactory,
		\Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollectionFactory,
		\Magento\Catalog\Model\ProductFactory $_productFactory,
		\Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollectionFactory
    ) {
        $this->logger = $logger;
		$this->configWriter = $configWriter;
		$this->_request = $request;
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_customerFactory = $customerFactory;
		$this->_orderCollectionFactory = $orderCollectionFactory;
		$this->_productCollectionFactory = $productCollectionFactory; 
        $this->_productFactory = $_productFactory;
    }

    public function execute(EventObserver $observer)
    {
		$isEnabled = $this->helper->getIsEnabled();
		if($isEnabled) {
			$uuid = $observer->getData('uuid');
			$scopeId = $observer->getData('scopeId');
			$scope = $observer->getData('scope');
			$apiUrl = $this->helper->getDataApiurl();
			$orderCollection = $this->_orderCollectionFactory->create()->addAttributeToSelect('*');
			$websiteId = $scopeId;	
			
			if($scope == \Magento\Store\Model\ScopeInterface::SCOPE_STORES) {
				$orderCollection = $orderCollection->addStoreFilter($scopeId);
			}
			
			if (!empty($orderCollection) && count($orderCollection)>0) {
				$o = 0;
				$i = 0;
				
				$total_orders = count($orderCollection);
				
				$mass_data = array(
					'uuid' => $uuid,
					'data' => array(
						array(
							'name' => 'Orders',
							'description' => 'Shop Orders'
						)
					)
				);

				foreach ($orderCollection as $order) {

					$payment = $order->getPayment();
					$method = $payment->getMethodInstance();
					$methodTitle = $method->getTitle();
						
					$mass_data['data'][0]['object'][$i] = array('order_details' => array(
						'customer_id' => $order->getCustomerId(),
						'order_id' => $order->getId(),
						'order_status' => $order->getStatus(),
						'shipping_method' => $order->getShippingMethod(),
						'payment_method' => $methodTitle,
						'total_price' => $order->getGrandTotal(),
						'order_create_date' => $order->getCreatedAt()
					));
					$orderItems = $order->getAllItems();
					foreach ($orderItems as $item) {
						$mass_data['data'][0]['object'][$i]['order_products'][] = array(
								'product_id' => $item->getProductId(),
								'quantity' => $item->getQty(),
						);			   
					}
					
					$i++;
					$o++;
			
					if ($i == 100 || $o == $total_orders) {
			
						$this->_curl->post($apiUrl, json_encode($mass_data));
						$this->_curl->addHeader("Content-Type", "application/json");
						$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
						//response will contain the output in form of JSON string
						
						$i = 0;
						$mass_data['data'][0]['object'] = array();
						
						if($o == $total_orders)
						{
							$response = $this->_curl->getBody();
							$responseArr = json_decode($response, true);
							if(!empty($responseArr['message'])) {
								$this->configWriter->save('waymoreroutee/general/datatransferred', $uuid, 'default', 0);
							}
						}
					}
				}
			}			
		}
		else {
		}
    }
    public function getOrderData($requestData)
    {
	$orderCollection = $this->_orderCollectionFactory->create();
	$orderCollection->addAttributeToSelect('*')->addAttributeToFilter("store_id", array("eq" => $requestData['store_id']));
	$apiUrl = $this->helper->getDataApiurl();
	
	if (!empty($orderCollection) && count($orderCollection)>0) {
		$o = 0;
		$i = 0;
		
		$total_orders = count($orderCollection);
		
		$mass_data = array(
			'uuid' => $requestData['uuid'],
			'data' => array(
				array(
					'name' => 'Orders',
					'description' => 'Shop Orders'
				)
			)
		);

		foreach ($orderCollection as $order) {

			$payment = $order->getPayment();
			$method = $payment->getMethodInstance();
			$methodTitle = $method->getTitle();
				
			$mass_data['data'][0]['object'][$i] = array('order_details' => array(
				'customer_id' => $order->getCustomerId(),
				'order_id' => $order->getId(),
				'order_status' => $order->getStatus(),
				'shipping_method' => $order->getShippingMethod(),
				'payment_method' => $methodTitle,
				'total_price' => $order->getGrandTotal(),
				'order_create_date' => $order->getCreatedAt()
			));
			$orderItems = $order->getAllItems();
			foreach ($orderItems as $item) {
				$mass_data['data'][0]['object'][$i]['order_products'][] = array(
						'product_id' => $item->getProductId(),
						'quantity' => $item->getQty(),
				);			   
			}
			
			$i++;
			$o++;
	
			if ($i == 100 || $o == $total_orders) {
	
				$this->_curl->post($apiUrl, json_encode($mass_data));
				$this->_curl->addHeader("Content-Type", "application/json");
				$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
				//response will contain the output in form of JSON string
				
				$i = 0;
				$mass_data['data'][0]['object'] = array();
				
				if($o == $total_orders)
				{
					$response = $this->_curl->getBody();
					$responseArr = json_decode($response, true);
					if(!empty($responseArr['message'])) {
						return "OrderDone";
					}
				}
			}
		}
	}
	else
	{
		return "OrderDone";
	}		
    }
}