<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Massapiproducts implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
	protected $_customerFactory;
	const CUSTOM_OPTION_STRING_PATH = 'waymoreroutee/general/uuid';

    /**
     * @param Logger $logger
     */
    public function __construct(
        Logger $logger,
		\Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
		\Magento\Framework\App\RequestInterface $request,
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager,
		\Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollectionFactory,
		\Magento\Catalog\Model\ProductFactory $_productFactory,
		\Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry
    ) {
        $this->logger = $logger;
		$this->configWriter = $configWriter;
		$this->_request = $request;
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_productCollectionFactory = $productCollectionFactory; 
        $this->_productFactory = $_productFactory;
		$this->_stockItem = $stockRegistry;
    }

    public function execute(EventObserver $observer)
    {
		$isEnabled = $this->helper->getIsEnabled();
		if($isEnabled) {
			$uuid = $observer->getData('uuid');
			$scopeId = $observer->getData('scopeId');
			$scope = $observer->getData('scope');
			$apiUrl = $this->helper->getDataApiurl();
			$productCollection = $this->_productCollectionFactory->create();
			$productCollection->addAttributeToSelect('*');
			$websiteId = $scopeId;	
			
			if($scope == \Magento\Store\Model\ScopeInterface::SCOPE_STORES) {
				$websiteId = $this->_storeManager->getStore($scopeId)->getWebsiteId();	
			}
			elseif($scope == \Magento\Store\Model\ScopeInterface::SCOPE_WEBSITES) {
				$websiteId = $scopeId;	
			}
			
			if($websiteId > 0) {
				$productCollection = $productCollection->getCollection()
					  ->addAttributeToSelect("*")
					  ->addAttributeToFilter("website_id", array("eq" => $websiteId)); 
			}
						
			$products = array();

			if (!empty($productCollection) && count($productCollection)>0) {
				$p = 0;
				$i = 0;
				
				$total_products = count($productCollection);

				$mass_data = array(
					'uuid' => $uuid,
					'data' => array(
						array(
							'name' => 'Products',
							'description' => 'Shop Products'
						)
					)
				);
				
				foreach ($productCollection as $product) {

					$discount = array();
						
					$specialprice = $product->getSpecialPrice();
					if($specialprice > 0) {
						$specialfromdate = $product->getSpecialFromDate();
						$specialtodate = $product->getSpecialToDate();
						$discount['price'] = $specialprice ;
						$discount['date_start'] = $specialfromdate ;
						$discount['date_end'] = $specialtodate ;
						$discount['quantity'] = null ;
					}

					$mass_data['data'][0]['object'][$i] = array(
						'product_id' => $product->getId(),
						'name' => $product->getName(),
						'description' => $product->getDescription(),
						'short_description' => $product->getShortDescription(),
						'stock_quantity' => $this->_stockItem->getStockItem($product->getId())->getQty(),
						'price' => $product->getPrice(),
						'discount' => $discount,
						'image_link' => $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . 'catalog/product' . $product->getImage(),
						'product_link' => $product->getProductUrl(),
					);
					$categories = $product->getCategoryIds(); /*will return category ids array*/
					if (!empty($categories)) {
						foreach($categories as $category){
							$cat = $this->_objectManager->create('Magento\Catalog\Model\Category')->load($category);
							$mass_data['data'][0]['object'][$i]['categories'][] = $cat->getName();
						}
					}
					$mass_data['data'][0]['object'][$i]['product_options'] = $this->_objectManager->get('Magento\Catalog\Model\Product\Option')->getProductOptionCollection($product);
					
					$i++;
					$p++;
			
					if ($i == 100 || $p == $total_products) {
			
						$this->_curl->post($apiUrl, json_encode($mass_data));
						$this->_curl->addHeader("Content-Type", "application/json");
						$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
						//response will contain the output in form of JSON string
						
						$i = 0;
						$mass_data['data'][0]['object'] = array();
						
						if($p == $total_products)
						{
							$response = $this->_curl->getBody();
							$responseArr = json_decode($response, true);
							if(!empty($responseArr['message'])) {
								$this->_eventManager->dispatch('waymoreroutee_massapiproduct_complete', ['uuid' => $uuid, 'scopeId' => $scopeId, 'scope' => $scope]);
							}
						}
					}
				}
			}
		}
		else {
		}
    }
    public function getProductData($requestData)
    {
		$websiteId = $this->_storeManager->getWebsite()->getId();

		$productCollection = $this->_productCollectionFactory->create();
		$productCollection->addAttributeToSelect('*');
		$apiUrl = $this->helper->getDataApiurl();
		
		if (!empty($productCollection) && count($productCollection)>0) {
			$p = 0;
			$i = 0;
			
			$total_products = count($productCollection);

			$mass_data = array(
				'uuid' => $requestData['uuid'],
				'data' => array(
					array(
						'name' => 'products',
						'description' => 'Shop Products'
					)
				)
			);
			
			foreach ($productCollection as $product) {

				$discount = array();
					
				$specialprice = $product->getSpecialPrice();
				if($specialprice > 0) {
					$specialfromdate = $product->getSpecialFromDate();
					$specialtodate = $product->getSpecialToDate();
					$discount['price'] = $specialprice ;
					$discount['date_start'] = $specialfromdate ;
					$discount['date_end'] = $specialtodate ;
					$discount['quantity'] = null ;
				}

				$mass_data['data'][0]['object'][$i] = array(
					'product_id' => $product->getId(),
					'name' => $product->getName(),
					'description' => $product->getDescription(),
					'short_description' => $product->getShortDescription(),
					'stock_quantity' => $this->_stockItem->getStockItem($product->getId())->getQty(),
					'price' => $product->getPrice(),
					'discount' => $discount,
					'image_link' => $this->_storeManager->getStore()->getBaseUrl(\Magento\Framework\UrlInterface::URL_TYPE_MEDIA) . 'catalog/product' . $product->getImage(),
					'product_link' => $product->getProductUrl(),
				);
				$categories = $product->getCategoryIds(); /*will return category ids array*/
				if (!empty($categories)) {
					foreach($categories as $category){
						$cat = $this->_objectManager->create('Magento\Catalog\Model\Category')->load($category);
						$mass_data['data'][0]['object'][$i]['categories'][] = $cat->getName();
					}
				}
				$mass_data['data'][0]['object'][$i]['product_options'] = $this->_objectManager->get('Magento\Catalog\Model\Product\Option')->getProductOptionCollection($product)->getData();
				
				$i++;
				$p++;
		
				if ($i == 100 || $p == $total_products) {
		
					$this->_curl->post($apiUrl, json_encode($mass_data));
					$this->_curl->addHeader("Content-Type", "application/json");
					$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
					//response will contain the output in form of JSON string
					
					$i = 0;
					$mass_data['data'][0]['object'] = array();
					
					if($p == $total_products)
					{
						$response = $this->_curl->getBody();
						$responseArr = json_decode($response, true);
						if(!empty($responseArr['message'])) {
							return "ProductDone";
						}
					}
				}
			}
		}
		else
		{
			return "ProductDone";
		}
    }
}