<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;

class Massapisubscribers implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
	protected $_customerFactory;
	const CUSTOM_OPTION_STRING_PATH = 'waymoreroutee/general/uuid';

    /**
     * @param Logger $logger
     */
    public function __construct(
        Logger $logger,
		\Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
		\Magento\Framework\App\RequestInterface $request,
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager,
		\Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerFactory,
		\Magento\Newsletter\Model\ResourceModel\Subscriber\CollectionFactory $subcriberCollectionFactory
    ) {
        $this->logger = $logger;
		$this->configWriter = $configWriter;
		$this->_request = $request;
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_customerFactory = $customerFactory;
		$this->_subcriberCollectionFactory = $subcriberCollectionFactory;
    }

    public function execute(EventObserver $observer)
    {
		$isEnabled = $this->helper->getIsEnabled();
		if($isEnabled) {
			$uuid = $observer->getData('uuid');
			$scopeId = $observer->getData('scopeId');
			$scope = $observer->getData('scope');
			$apiUrl = $this->helper->getDataApiurl();
			$subscriberCollection = $this->_subcriberCollectionFactory->create();
			$websiteId = $scopeId;	
			
			if($scope == \Magento\Store\Model\ScopeInterface::SCOPE_STORES) {
				$subscriberCollection = $subscriberCollection->addStoreFilter($scopeId);
			}
			
			if (!empty($subscriberCollection) && count($subscriberCollection)>0) {
				$s = 0;
				$i = 0;
				
				$total_subscribers = count($subscriberCollection);

				$mass_data = array(
					'uuid' => $uuid,
					'data' => array(
						array(
							'name' => 'Subscribers',
							'description' => 'Shop Subscribers'
						)
					)
				);
				
				foreach ($subscriberCollection as $subscriber) {

					$mass_data['data'][0]['object'][$i] = array('subscriber' => array(
						'id' => $subscriber->getSubscriberId(),
						'id_shop' => $scopeId,
						'id_shop_group' => $scopeId,
						'email' => $subscriber->getSubscriberEmail(),
						'newsletter_date_add' => $subscriber->getChangeStatusAt(),
						'ip_registration_newsletter' => 'N/A',
						'http_referer' => 'N/A',
						'active' => $subscriber->getSubscriberStatus(),
					));

					$i++;
					$s++;
			
					if ($i == 100 || $s == $total_subscribers) {
			
						$this->_curl->post($apiUrl, json_encode($mass_data));
						$this->_curl->addHeader("Content-Type", "application/json");
						$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
						//response will contain the output in form of JSON string
						
						$i = 0;
						$mass_data['data'][0]['object'] = array();
						
						if($s == $total_subscribers)
						{
							$response = $this->_curl->getBody();
							$responseArr = json_decode($response, true);
							if(!empty($responseArr['message'])) {
								$this->_eventManager->dispatch('waymoreroutee_massapisubscriber_complete', ['uuid' => $uuid, 'scopeId' => $scopeId, 'scope' => $scope]);
							}
						}
					}
				}
			}
		}
		else {
		}
    }
    public function getSubscriberData($requestData)
    {
	$subscriberCollection = $this->_subcriberCollectionFactory->create();
	$subscriberCollection->addStoreFilter($requestData['store_id']);
	$apiUrl = $this->helper->getDataApiurl();

	if (!empty($subscriberCollection) && count($subscriberCollection)>0) {
		$s = 0;
		$i = 0;
		
		$total_subscribers = count($subscriberCollection);

		$mass_data = array(
			'uuid' => $requestData['uuid'],
			'data' => array(
				array(
					'name' => 'Subscribers',
					'description' => 'Shop Subscribers'
				)
			)
		);
		
		foreach ($subscriberCollection as $subscriber) {

			$mass_data['data'][0]['object'][$i] = array('subscriber' => array(
				'id' => $subscriber->getSubscriberId(),
				'id_shop' => $requestData['store_id'],
				'id_shop_group' => $requestData['store_id'],
				'email' => $subscriber->getSubscriberEmail(),
				'newsletter_date_add' => $subscriber->getChangeStatusAt(),
				'ip_registration_newsletter' => 'N/A',
				'http_referer' => 'N/A',
				'active' => $subscriber->getSubscriberStatus(),
			));

			$i++;
			$s++;
	
			if ($i == 100 || $s == $total_subscribers) {
	
				$this->_curl->post($apiUrl, json_encode($mass_data));
				$this->_curl->addHeader("Content-Type", "application/json");
				$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
				//response will contain the output in form of JSON string
				
				$i = 0;
				$mass_data['data'][0]['object'] = array();
				
				if($s == $total_subscribers)
				{
					$response = $this->_curl->getBody();
					$responseArr = json_decode($response, true);
					if(!empty($responseArr['message'])) {
						return "SubscriberDone";
					}
				}
			}
		}
	}
	else
	{
		return "SubscriberDone";
	}		
    }
}