<?php
namespace Routee\WaymoreRoutee\Observer;

use Magento\Framework\Event\ObserverInterface;
use Magento\Framework\Event\Observer as EventObserver;
use Psr\Log\LoggerInterface as Logger;
use Magento\Framework\App\Config\ScopeConfigInterface;

use Magento\Wishlist\Controller\WishlistProvider;
use Magento\Wishlist\Model\ResourceModel\Item\CollectionFactory;
use Magento\Wishlist\Model\WishlistFactory;

class Massapiwishlists implements ObserverInterface
{
    /**
     * @var Logger
     */
    protected $logger;
	protected $_customerFactory;
	const CUSTOM_OPTION_STRING_PATH = 'waymoreroutee/general/uuid';

    /**
     * @param Logger $logger
     */
    public function __construct(
        Logger $logger,
		\Magento\Framework\App\Config\Storage\WriterInterface $configWriter,
		\Magento\Framework\App\RequestInterface $request,
		\Routee\WaymoreRoutee\Helper\Data $helper,
		\Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
		\Magento\Framework\App\Config\ConfigResource\ConfigInterface $configInterface,
		\Magento\Framework\HTTP\Client\Curl $curl,
		\Magento\Framework\ObjectManagerInterface $objectmanager,
		\Magento\Framework\Module\ResourceInterface $moduleResource,
		\Magento\Store\Model\StoreManagerInterface $storeManager,
		\Magento\Framework\Event\Manager $eventManager,
		\Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerFactory,
		CollectionFactory $wishlistCollectionFactory,
		\Magento\Wishlist\Model\WishlistFactory $wishlistRepository,
        WishlistFactory $wishlistFactory
    ) {
        $this->logger = $logger;
		$this->configWriter = $configWriter;
		$this->_request = $request;
		$this->helper = $helper;
		$this->scopeConfig = $scopeConfig;
		$this->_configInterface = $configInterface;
		$this->_curl = $curl;
		$this->_objectManager = $objectmanager;
		$this->moduleResource = $moduleResource;
		$this->_storeManager = $storeManager;
		$this->_eventManager = $eventManager;
		$this->_customerFactory = $customerFactory;
		$this->_wishlistCollectionFactory = $wishlistCollectionFactory;
        $this->_wishlistRepository = $wishlistRepository;
		$this->_wishlistFactory = $wishlistFactory;
    }

    public function execute(EventObserver $observer)
    {
		$isEnabled = $this->helper->getIsEnabled();
		if($isEnabled) {
			$uuid = $observer->getData('uuid');
			$scopeId = $observer->getData('scopeId');
			$scope = $observer->getData('scope');
			$apiUrl = $this->helper->getDataApiurl();
			$wishlistCollection = $this->_wishlistCollectionFactory->create();
			$websiteId = $scopeId;	
			
			if($scope == \Magento\Store\Model\ScopeInterface::SCOPE_STORES) {
				$wishlistCollection = $wishlistCollection->addStoreFilter($scopeId);
			}
			
			if (!empty($wishlistCollection) && count($wishlistCollection)>0) {
				$w = 0;
				$i = 0;
				
				$total_wishlists = count($wishlistCollection);

				$mass_data = array(
					'uuid' => $uuid,
					'data' => array(
						array(
							'name' => 'Wishlists',
							'description' => 'Shop Wishlists'
						)
					)
				);

				foreach ($wishlistCollection as $wishlist) {

					$mass_data['data'][0]['object'][$i] = array(
						'customer_id' => $wishlist->getCustomerId(),
						'product_id' => $wishlist->getProductId()
					);

					$i++;
					$w++;
			
					if ($i == 100 || $w == $total_wishlists) {
			
						$this->_curl->post($apiUrl, json_encode($mass_data));
						$this->_curl->addHeader("Content-Type", "application/json");
						$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
						//response will contain the output in form of JSON string
						
						$i = 0;
						$mass_data['data'][0]['object'] = array();
						
						if($w == $total_wishlists)
						{
							$response = $this->_curl->getBody();
							$responseArr = json_decode($response, true);
							if(!empty($responseArr['message'])) {
								$this->_eventManager->dispatch('waymoreroutee_massapiwishlist_complete', ['uuid' => $uuid, 'scopeId' => $scopeId, 'scope' => $scope]);
							}
						}
					}
				}
			}			
		}
		else {
		}
    }
    public function getWishlistData($requestData)
    {
		$wishlistCollection = $this->_wishlistCollectionFactory->create();
		$wishlistCollection->addStoreFilter($requestData['store_id']);
		$apiUrl = $this->helper->getDataApiurl();
		
		if (!empty($wishlistCollection) && count($wishlistCollection)>0) {
			$w = 0;
			$i = 0;
			
			$total_wishlists = count($wishlistCollection);

			$mass_data = array(
				'uuid' => $requestData['uuid'],
				'data' => array(
					array(
						'name' => 'Wishlists',
						'description' => 'Shop Wishlists'
					)
				)
			);

			foreach ($wishlistCollection as $wishlist) {

				$mass_data['data'][0]['object'][$i] = array(
					'customer_id' => $wishlist->getCustomerId(),
					'product_id' => $wishlist->getProductId()
				);

				$i++;
				$w++;
		
				if ($i == 100 || $w == $total_wishlists) {
		
					$this->_curl->post($apiUrl, json_encode($mass_data));
					$this->_curl->addHeader("Content-Type", "application/json");
					$this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
					//response will contain the output in form of JSON string
					
					$i = 0;
					$mass_data['data'][0]['object'] = array();
					
					if($w == $total_wishlists)
					{
						$response = $this->_curl->getBody();
						$responseArr = json_decode($response, true);
						if(!empty($responseArr['message'])) {
							return "WishlistDone";
						}
					}
				}
			}
		}
		else
		{
			return "WishlistDone";
		}		
    }
}