<?php 
namespace Routee\WaymoreRoutee\Setup;

use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Framework\Db\Select;
use Magento\Framework\Setup\ModuleContextInterface;
use Magento\Framework\Setup\SchemaSetupInterface;
use Magento\Framework\Setup\UninstallInterface as UninstallInterface;
use Magento\Eav\Setup\EavSetupFactory;
use Magento\Framework\Setup\ModuleDataSetupInterface;


/**
 * Class Uninstall
 */
class Uninstall implements UninstallInterface
{
    /**
     * EAV setup factory
     *
     * @var EavSetupFactory
     */
    private $_eavSetupFactory;
    
    private $_mDSetup;
    
    /**
     * @var \Magento\Framework\App\Config\ConfigResource\ConfigInterface
     */
    protected $resourceConfig;

    /**
    * @var \Magento\Store\Model\StoreManagerInterface
    * This variable value asisgned below in constructor
    * Added by Kapil on 20th April 2020
    */
    protected $_storeManager;
    /**
     * Init
     *
     * @param EavSetupFactory $eavSetupFactory
     */
    public function __construct(
        EavSetupFactory $eavSetupFactory,
        ModuleDataSetupInterface $mDSetup,
        \Magento\Config\Model\ResourceModel\Config $resourceConfig,
        \Routee\WaymoreRoutee\Helper\Data $helper,
        \Magento\Framework\HTTP\Client\Curl $curl,
        \Magento\Store\Model\StoreManagerInterface $storeManager
    )
    {
        $this->helper = $helper;
        $this->_eavSetupFactory = $eavSetupFactory;
        $this->_mDSetup = $mDSetup;
        $this->resourceConfig = $resourceConfig;
        $this->_curl = $curl;
        //Added by Kapil on 20th April 2020
        $this->_storeManager = $storeManager;
    }

    public function uninstall(SchemaSetupInterface $setup, ModuleContextInterface $context)
    {
        $storeId = $this->_storeManager->getStore()->getId();   
        $uuid = $this->helper->getUuid($storeId);
        
        /* 
        * Added to remove data from routee database when calling uninstall event
        * Also added curl library in constructor to use below.
        * Created on 26th Feb 2020
        * By : Kapil Hanswal
        */
        $eventdata = array();
        $eventdata['data'] = [];
        $eventdata['uuid'] = $uuid;
        $eventdata['event'] = "Uninstall";

        $url = $this->helper->getEventApiurl();

        $this->_curl->post($url, json_encode($eventdata));
        $this->_curl->addHeader("Content-Type", "application/json");
        $this->_curl->setOption(CURLOPT_RETURNTRANSFER, true);
        /*Ends here*/

        /**
        * Below code deleting config details from the database so next time after reinstallation user have to submit again to generate UUID
        * Initially it was not removing due to wrong config library but included correct one
        * 
        * Removed all the unnecessary code from this script
        * Added by Kapil on 21st April 2020
        */
        $this->resourceConfig->deleteConfig('waymoreroutee/general/enable');
        $this->resourceConfig->deleteConfig('waymoreroutee/general/username');
        $this->resourceConfig->deleteConfig('waymoreroutee/general/password');
        $this->resourceConfig->deleteConfig('waymoreroutee/general/uuid');
        $this->resourceConfig->deleteConfig('waymoreroutee/general/testmode');
        $this->resourceConfig->deleteConfig('waymoreroutee/general/datatransferred');        
    }
}
